/*
Checking program for task F: Triples
Author: Piotr Zielinski & Bartosz Nowierski
*/

#include <stdio.h>
#include <string.h>
#include <assert.h>

#define assertionfailed assert(0)

inline int min(int a, int b) {
  if (a<b) return a;
  else return b;
}

inline int max(int a, int b) {
  if (a<b) return b;
  else return a;
}

inline int vectorproduct(int dx1, int dy1, int dx2, int dy2) {
  return dx1*dy2 - dx2*dy1;
}

int segmentscross(int ax1, int ay1, int ax2, int ay2, 
		  int bx1, int by1, int bx2, int by2) 
{
  //Preliminary checks

  if (min(ax1, ax2) > max(bx1, bx2)) return 0;
  if (max(ax1, ax2) < min(bx1, bx2)) return 0;
  if (min(ay1, ay2) > max(by1, by2)) return 0;
  if (max(ay1, ay2) < min(by1, by2)) return 0;

  //Proper tests

  int vec1, vec2;

  vec1 = vectorproduct(ax2 - ax1, ay2 - ay1,
		       bx1 - ax1, by1 - ay1);
  vec2 = vectorproduct(ax2 - ax1, ay2 - ay1,
		       bx2 - ax1, by2 - ay1);

  if (vec1 < 0 && vec2 < 0) return 0;
  if (vec1 > 0 && vec2 > 0) return 0;

  vec1 = vectorproduct(bx2 - bx1, by2 - by1,
		       ax1 - bx1, ay1 - by1);
  vec2 = vectorproduct(bx2 - bx1, by2 - by1,
		       ax2 - bx1, ay2 - by1);

  if (vec1 < 0 && vec2 < 0) return 0;
  if (vec1 > 0 && vec2 > 0) return 0;

  return 1;
}

const int MAXTRIANGLES = 10000;
int triangles;

int X[MAXTRIANGLES][4];
int Y[MAXTRIANGLES][4];


int load(FILE *fin, FILE *fout)
{
  int XX[MAXTRIANGLES*3], YY[MAXTRIANGLES*3];
  int used[MAXTRIANGLES*3];


  if (fscanf(fin, "%i", &triangles) < 1)
    return -1;
  memset(used, 0, sizeof(used[0])*triangles*3);

  if (!(2 <= triangles && triangles <= MAXTRIANGLES))
    return -1;

   for(int i=0; i<3*triangles; i++)
   {
     if (fscanf(fin, "%i%i",  &XX[i], &YY[i]) < 2)
       return -1;
     if (XX[i]<0 || XX[i]>10000 | YY[i]<0 || YY[i] >10000)
       return -1;
   }
      
   for(int i=0; i<triangles; i++)
   {
     for (int j=0; j<3; j++)
     {
       int p;
       if (fscanf(fout, "%i", &p) < 1)
         return 0;
       if (p<1 || p>3*triangles)
         return 0;
       p--;
       if (used[p])
         return 0;
       used[p]=1;
       X[i][j] = XX[p];
       Y[i][j] = YY[p];
     }
     X[i][3] = X[i][0];
     Y[i][3] = Y[i][0];
   }

  return 1;
}


int solve(void) {
  for(int i=0; i<triangles; i++)
    for(int j=0; j<i; j++)
    {
      for(int k=0; k<3; k++)
      for(int l=0; l<3; l++)
  	  if (segmentscross(X[i][k], Y[i][k], X[i][k+1], Y[i][k+1],
			    X[j][l], Y[j][l], X[j][l+1], Y[j][l+1]))
      {
        return 1;
      }
	  }
  return 0;
}


int main(int argc, char *argv[])
{
  char finname[100], foutname[100];
  FILE *fin = stdin;
  FILE *fout = stdout;
  int cases;


  if (argc<=2)
    return 255;
   
  if (snprintf(finname, 100, "%s/input", argv[1]) < 0)
    return 255;
  if (snprintf(foutname, 100, "%s/output", argv[2]) < 0)
    return 255;
  fin = fopen(finname, "r");
  if (fin==NULL)
    return 255;
  fout = fopen(foutname, "r");
  if (fout==NULL)
    return 255;

  

  if (fscanf(fin, "%i", &cases) < 1)
    return 255;
  
  for(int i=0; i<cases; i++)
  {
    int r;
    r=load(fin,fout);
    if (r<0)
      return 255;
    if (r==0)
      return 1;
//    fprintf(fout, "%s\n", solve()?"YES":"NO");
    if (solve())
    {
      printf("iter: %d\n", i);
      return 1;
    }
  }

  return 0;
}
    
